'
' This file contains Implementations of a Mode 0, 1, 2, and 3 master SPI device.
'
    Dim PP0 As Byte System                  ' Use a compiler system variable for the transfer variable
    Dim PP0H As Byte System                 ' Use a compiler system variable for the loop counter   

$ifndef SPI_CS                              ' \
    $define SPI_CS PORTB.0                  ' |
$endif                                      ' |
$ifndef SPI_SCK                             ' |
    $define SPI_SCK PORTB.1                 ' |
$endif                                      ' | Set the IO lines For SPI Mode 0 routine
$ifndef SPI_SDO                             ' |
    $define SPI_SDO PORTB.2                 ' | 
$endif                                      ' |
$ifndef SPI_SDI                             ' |
    $define SPI_SDI PORTB.3                 ' /
$endif

Reminders = Off
Small_Micro_Model = On
'-----------------------------------------------------------------------------
    GoTo Init_Software_SPI                  ' Jump over the subroutines
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one Byte <PP0> using the SPI mode 0 protocol
' SCK is idle-low, and bits are latched On SCK rising.
'
SPI_Out0 Macro pBytein
    #if(Prm_Count >= 1)
        #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
            Num_Wreg pBytein
        #else
            Byte_Wreg pBytein
        #endif
    #else
        movlw 0
    #endif
    GoSub SPIMode_0
    #if(SPI_Out0_Return == 1)
        #if(Return_Type == Byte)
            Wreg_Byte Return_Var
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
    #endif
Endm

#ifdef SPI_Out0#Req
'
' Input     : WREG = 8-bit value to send
'           : SPI_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI_SDO holds the Port.Bit of the SPI Data In Pin
'           : SPI_SDI holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-bit value received
'
SPIMode_0:
    Clear SPI_CS
    $if (_core == 16)
        movwf PP0
    $else
        Wreg_Byte PP0                   ' Transfer the WREG into PP0
    $endif
    PP0H = 8                            ' Single Byte SPI loop
SpiMode0Lp:
        Clear SPI_SDO                   ' \
        If PP0.7 = 1 Then               ' |
            Set SPI_SDO                 ' | Put current outgoing bit On SDO
        EndIf                           ' /
        PP0 = PP0 << 1                  ' Shift next bit into MSB
        Set SPI_SCK                     ' Set SCK high
        PP0 = PP0 | SPI_SDI             ' Capture current bit On SDI
        Clear SPI_SCK                   ' Pull SCK low
    djnz PP0H,SpiMode0Lp
    $if (_core == 16)
        movf PP0,W                      ' Transfer PP0 into WREG
    $else
        Byte_Wreg PP0
    $endif
    Set SPI_CS
    Return
#endif
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one byte <PP0> using the SPI mode 1 protocol.
' SCK is idle-low, and bits are latched on SCK falling.
'
SPI_Out1 Macro pBytein
    #if(Prm_Count >= 1)
        #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
            Num_Wreg pBytein
        #else
            Byte_Wreg pBytein
        #endif
    #else
        movlw 0
    #endif
    GoSub SPIMode_1
    #if(SPI_Out1_Return == 1)
        #if(Return_Type == Byte)
            Wreg_Byte Return_Var
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
    #endif
Endm

#ifdef SPI_Out1#Req
'
' Input     : WREG = 8-bit value to send
'           : SPI_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI_SDO holds the Port.Bit of the SPI Data In Pin
'           : SPI_SDI holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-bit value received
'
SPIMode_1:
    Clear SPI_CS
    $if (_core = 16)
        movwf PP0
    $else
        Wreg_Byte PP0                   ' Transfer the WREG into PP0
    $endif
    PP0H = 8                            ' Single Byte SPI loop
SpiMode1Lp:
        Set SPI_SCK                     ' Set SCK high
        Clear SPI_SDO                   ' \
        If PP0.7 = 1 Then               ' |
            Set SPI_SDO                 ' | Put current outgoing bit On SDO
        EndIf                           ' /
        PP0 = PP0 << 1                  ' Shift next bit into MSB
        Clear SPI_SCK                   ' Pull SCK low
        PP0 = PP0 | SPI_SDI             ' Capture current bit On SDI
    djnz PP0H,SpiMode1Lp
    $if (_core = 16)
        movf PP0,W                      ' Transfer PP0 into WREG
    $else
        Byte_Wreg PP0
    $endif
    Set SPI_CS
    Return
#endif
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one byte <PP0> using the SPI mode 2 protocol.
' SCK is idle-high, and bits are latched on SCK falling.
'
SPI_Out2 Macro pBytein
    #if(Prm_Count >= 1)
        #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
            Num_Wreg pBytein
        #else
            Byte_Wreg pBytein
        #endif
    #else
        movlw 0
    #endif
    GoSub SPIMode_2
    #if(SPI_Out2_Return == 1)
        #if(Return_Type == Byte)
            Wreg_Byte Return_Var
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
    #endif
Endm

#ifdef SPI_Out2#Req
'
' Input     : WREG = 8-bit value to send
'           : SPI_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI_SDO holds the Port.Bit of the SPI Data In Pin
'           : SPI_SDI holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-bit value received
'
SPIMode_2:
    Clear SPI_CS
    $if (_core = 16)
        movwf PP0
    $else
        Wreg_Byte PP0                   ' Transfer the WREG into PP0
    $endif
    PP0H = 8                            ' Single Byte SPI loop
SpiMode2Lp:
        Clear SPI_SDO                   ' \
        If PP0.7 = 1 Then               ' |
            Set SPI_SDO                 ' | Put current outgoing bit On SDO
        EndIf                           ' /
        PP0 = PP0 << 1                  ' Shift next bit into MSB
        Clear SPI_SCK                   ' Set SCK high
        PP0 = PP0 | SPI_SDI             ' Capture current bit On SDI
        Set SPI_SCK                     ' Pull SCK low
    djnz PP0H,SpiMode2Lp
    $if (_core = 16)
        movf PP0,W                      ' Transfer PP0 into WREG
    $else
        Byte_Wreg PP0
    $endif
    Set SPI_CS
    Return
#endif
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one byte <PP0> using the SPI mode 3 protocol
' SCK is idle-high, and bits are latched on SCK rising.
'
SPI_Out3 Macro pBytein
    #if(Prm_Count >= 1)
        #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
            Num_Wreg pBytein
        #else
            Byte_Wreg pBytein
        #endif
    #else
        movlw 0
    #endif
    GoSub SPIMode_3
    #if(SPI_Out3_Return == 1)
        #if(Return_Type == Byte)
            Wreg_Byte Return_Var
        #endif
        #if(Return_Type == Word)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword)
            Wreg_Dword Return_Var
        #endif
    #endif
Endm

#ifdef SPI_Out3#Req
'
' Input     : WREG = 8-bit value to send
'           : SPI_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI_SDO holds the Port.Bit of the SPI Data In Pin
'           : SPI_SDI holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-bit value received
'
SPIMode_3:
    Clear SPI_CS
    $if (_core = 16)
        movwf PP0
    $else
        Wreg_Byte PP0                   ' Transfer the WREG into PP0
    $endif
    PP0H = 8                            ' Single Byte SPI loop
SpiMode3Lp:
        Clear SPI_SCK                   ' Pull SCK low
        Clear SPI_SDO                   ' \
        If PP0.7 = 1 Then               ' |
            Set SPI_SDO                 ' | Put current outgoing bit On SDO
        EndIf                           ' /
        PP0 = PP0 << 1                  ' Shift next bit into MSB
        Set SPI_SCK                     ' Set SCK high
        PP0  = PP0 | SPI_SDI            ' Capture current bit On SDI
    djnz PP0H,SpiMode3Lp
    $if (_core = 16)
        movf PP0,W                      ' Transfer PP0 into WREG
    $else
        Byte_Wreg PP0
    $endif
    Set SPI_CS
    Return
#endif
Small_Micro_Model = Off
Reminders = On

'-----------------------------------------------------------------------------
Init_Software_SPI:

    High SPI_CS
    Output SPI_SCK
    Output SPI_SDO
    Input SPI_SDI

